@@ -100,6 +100,9 @@ ControlOSC::ControlOSC(Engine * eng, unsigned int port)
 
 	// for all loops
 	on_loop_added(-1);
+
+	// for selected loop
+	on_loop_added(-3);
 	
 	// lo_server_thread_add_method(_sthread, NULL, NULL, ControlOSC::_dummy_handler, this);
 
@@ -142,8 +145,9 @@ ControlOSC::register_callbacks()
 		/* add method that will match the path /quit with no args */
 		lo_server_add_method(serv, "/quit", "", ControlOSC::_quit_handler, this);
 
-		// add ping handler:  s:returl s:retpath
+		// add ping handler:  s:returl s:retpath [useid]
 		lo_server_add_method(serv, "/ping", "ss", ControlOSC::_ping_handler, this);
+		lo_server_add_method(serv, "/ping", "ssi", ControlOSC::_ping_handler, this);
 
 		// add loop add handler:  i:channels  i:bytes_per_channel
 		lo_server_add_method(serv, "/loop_add", "if", ControlOSC::_loop_add_handler, this);
@@ -152,8 +156,9 @@ ControlOSC::register_callbacks()
 		// load session:  s:filename  s:returl  s:retpath
 		lo_server_add_method(serv, "/load_session", "sss", ControlOSC::_load_session_handler, this);
 
-		// save session:  s:filename  s:returl  s:retpath
+		// save session:  s:filename  s:returl  s:retpath (i:write_audio)
 		lo_server_add_method(serv, "/save_session", "sss", ControlOSC::_save_session_handler, this);
+		lo_server_add_method(serv, "/save_session", "sssi", ControlOSC::_save_session_handler, this);
 		
 		// add loop del handler:  i:index 
 		lo_server_add_method(serv, "/loop_del", "i", ControlOSC::_loop_del_handler, this);
@@ -257,7 +262,7 @@ ControlOSC::terminate_osc_thread ()
 	void* status;
 
 	_shutdown = true;
-
+	
 	poke_osc_thread ();
 
 	pthread_join (_osc_thread, &status);
@@ -439,7 +444,7 @@ ControlOSC::osc_receiver()
 		if ((ret = poll (pfd, nfds, timeout)) < 0) {
 			if (errno == EINTR) {
 				/* gdb at work, perhaps */
-				//cerr << "EINTR hit " << endl;
+				cerr << "EINTR hit " << endl;
 				goto again;
 			}
 			
@@ -464,15 +469,20 @@ ControlOSC::osc_receiver()
 			{
 				// this invokes callbacks
 				//cerr << "invoking recv on " << pfd[i].fd << endl;
-				lo_server_recv (srvs[i]);
+				lo_server_recv(srvs[i]);
 			}
 		}
 
 	}
 
-	cerr << "SL engine shutdown" << endl;
+	//cerr << "SL engine shutdown" << endl;
 	
 	if (_osc_server) {
+		int fd = lo_server_get_socket_fd(_osc_server);
+		if (fd >=0) {
+				// hack around
+			close(fd);
+		}
 		lo_server_free (_osc_server);
 		_osc_server = 0;
 	}
@@ -483,6 +493,8 @@ ControlOSC::osc_receiver()
 		_osc_unix_server = 0;
 	}
 	
+	close(_request_pipe[0]);
+	close(_request_pipe[1]);
 }
 
 
@@ -699,6 +711,10 @@ int ControlOSC::ping_handler(const char *path, const char *types, lo_arg **argv,
 {
 	string returl (&argv[0]->s);
 	string retpath (&argv[1]->s);
+	bool useid = false;
+	if (argc > 2) {
+		useid = true;
+	}
 
 	//lo_message msg = (lo_message) data;
 	//lo_address srcaddr = lo_message_get_source (msg);
@@ -706,7 +722,7 @@ int ControlOSC::ping_handler(const char *path, const char *types, lo_arg **argv,
 	//int srcport = atoi(sport);
 	//cerr << "got ping from " <<  srcport << endl;
 
-	_engine->push_nonrt_event ( new PingEvent (returl, retpath));
+	_engine->push_nonrt_event ( new PingEvent (returl, retpath, useid));
 	
 	return 0;
 }
@@ -993,9 +1009,14 @@ int ControlOSC::save_session_handler(const char *path, const char *types, lo_arg
 	string fname (&argv[0]->s);
 	string returl (&argv[1]->s);
 	string retpath (&argv[2]->s);
+	bool audio = false;
+
+	if (argc > 3) {
+		audio = (bool) argv[3]->i;
+	}
 
 	// push this onto a queue for the main event loop to process
-	_engine->push_nonrt_event ( new SessionEvent (SessionEvent::Save, fname, returl, retpath));
+	_engine->push_nonrt_event ( new SessionEvent (SessionEvent::Save, fname, returl, retpath, audio));
 	
 	return 0;
 }
@@ -1405,7 +1426,8 @@ void ControlOSC::send_auto_updates ()
 		_last_value_map[ipair] = val;
 
 		// cerr << "ctrl " << ipair.second << " is new: " << val << endl;
-		
+		//_engine->ParamChanged(_cmd_map->to_control_t(ipair.second), ipair.first);
+
 		if ( ! send_registered_updates (iter, ipair.second, val, ipair.first, -1)) {
 			// remove ipair if false is returned.. no more good registrations
 			tmpiter = iter;
@@ -1478,7 +1500,7 @@ void ControlOSC::send_all_config ()
 	// for now just send pingacks to all registered addresses
 	for (AddressList::iterator iter = _config_registrations.begin(); iter != _config_registrations.end(); ++iter)
 	{
-		send_pingack (true, (*iter).first, (*iter).second);
+		send_pingack (true, false, (*iter).first, (*iter).second);
 	}
 }
 
@@ -1498,7 +1520,7 @@ void ControlOSC::send_error (std::string returl, std::string retpath, std::strin
 	}
 }
 
-void ControlOSC::send_pingack (bool useudp, string returl, string retpath)
+void ControlOSC::send_pingack (bool useudp, bool use_id, string returl, string retpath)
 {
 	lo_address addr;
 
@@ -1519,8 +1541,17 @@ void ControlOSC::send_pingack (bool useudp, string returl, string retpath)
 	}
 	
 	//cerr << "sooperlooper: sending ping response to " << returl << endl;
-	if (lo_send(addr, retpath.c_str(), "ssi", oururl.c_str(), sooperlooper_version, _engine->loop_count()) < 0) {
-		fprintf(stderr, "OSC error %d: %s\n", lo_address_errno(addr), lo_address_errstr(addr));
+	// sends our server URL, the SL version, the loop count, and a unique id for continuity checking
+	if (use_id)
+	{
+		if (lo_send(addr, retpath.c_str(), "ssii", oururl.c_str(), sooperlooper_version, _engine->loop_count(), _engine->get_id()) < 0) {
+			fprintf(stderr, "OSC error %d: %s\n", lo_address_errno(addr), lo_address_errstr(addr));
+		}
+	}
+	else {
+		if (lo_send(addr, retpath.c_str(), "ssi", oururl.c_str(), sooperlooper_version, _engine->loop_count()) < 0) {
+			fprintf(stderr, "OSC error %d: %s\n", lo_address_errno(addr), lo_address_errstr(addr));
+		}
 	}
 }
 
