@@ -39,33 +39,45 @@ using namespace SooperLooper;
 #define DEFAULT_OSC_PORT 9951
 
 LoopControl::SpawnConfig::SpawnConfig(const wxString & nm)
-	: name(nm), host(wxT("")), port(DEFAULT_OSC_PORT), num_loops(1), num_channels(2),
-	  mem_secs(40.0f), discrete_io(true), exec_name(wxT("sooperlooper")), midi_bind_path(wxT("")), force_spawn(false), never_spawn(false),
-	  jack_name(wxT(""))
+	: name(nm), host(""), port(DEFAULT_OSC_PORT), num_loops(1), num_channels(2),
+	  mem_secs(40.0f), discrete_io(true), exec_name("sooperlooper"), midi_bind_path(""), force_spawn(false), never_spawn(false),
+	  never_timeout(false),
+	  jack_name("")
 {
 }
 
 
 XMLNode& LoopControl::SpawnConfig::get_state () const
 {
+	char buf[40];
 	XMLNode *node = new XMLNode ("context");
 
-	node->add_property ("name", name.ToAscii());
+	node->add_property ("name", (const char *) name.ToAscii());
+
+	node->add_property ("host", host);
+
+	snprintf(buf, sizeof(buf), "%ld", port);
+	node->add_property ("port", buf);
+
+	snprintf(buf, sizeof(buf), "%ld", num_loops);
+	node->add_property ("num_loops", buf);
+
+	snprintf(buf, sizeof(buf), "%ld", num_channels);
+	node->add_property ("num_channels", buf);
+
+	snprintf(buf, sizeof(buf), "%f", mem_secs);
+	node->add_property ("mem_secs", buf);
 
-	node->add_property ("host", host.ToAscii());
-	node->add_property ("port", wxString::Format(wxT("%ld"), port).ToAscii());
-	node->add_property ("num_loops", wxString::Format(wxT("%ld"), num_loops).ToAscii());
-	node->add_property ("num_channels", wxString::Format(wxT("%ld"), num_channels).ToAscii());
-	node->add_property ("mem_secs", wxString::Format(wxT("%f"), mem_secs).ToAscii());
 	node->add_property ("discrete_io", discrete_io ? "yes": "no");
 
-	node->add_property ("exec_name", exec_name.ToAscii());
-	node->add_property ("jack_name", jack_name.ToAscii());
-	node->add_property ("jack_serv_name", jack_serv_name.ToAscii());
-	node->add_property ("midi_bind_path", midi_bind_path.ToAscii());
-	node->add_property ("session_path", session_path.ToAscii());
+	node->add_property ("exec_name", exec_name);
+	node->add_property ("jack_name", jack_name);
+	node->add_property ("jack_serv_name", jack_serv_name);
+	node->add_property ("midi_bind_path", midi_bind_path);
+	node->add_property ("session_path", session_path);
 
 	node->add_property ("force_spawn", force_spawn ? "yes": "no");
+	node->add_property ("never_timeout", never_timeout ? "yes": "no");
 	
 	return *node;
 }
@@ -85,26 +97,22 @@ int LoopControl::SpawnConfig::set_state (const XMLNode& node)
 			XMLProperty *prop;
 			
 			if ((prop = child_node->property ("name")) != 0) {
-				name.Printf(wxT("%s"), prop->value().c_str());
+				name =  wxString::FromAscii(prop->value().c_str());
 			}
 			if ((prop = child_node->property ("host")) != 0) {
-				host.Printf(wxT("%s"), prop->value().c_str());
+				host = prop->value();
 			}
 			if ((prop = child_node->property ("port")) != 0) {
-				tmpstr.Printf(wxT("%s"), prop->value().c_str());
-				tmpstr.ToLong(&port);
+				port = atol (prop->value().c_str());
 			}
 			if ((prop = child_node->property ("num_loops")) != 0) {
-				tmpstr.Printf(wxT("%s"), prop->value().c_str());
-				tmpstr.ToLong(&num_loops);
+				num_loops = atol (prop->value().c_str());
 			}
 			if ((prop = child_node->property ("num_channels")) != 0) {
-				tmpstr.Printf(wxT("%s"), prop->value().c_str());
-				tmpstr.ToLong(&num_channels);
+				num_channels = atol (prop->value().c_str());
 			}
 			if ((prop = child_node->property ("mem_secs")) != 0) {
-				tmpstr.Printf(wxT("%s"), prop->value().c_str());
-				tmpstr.ToDouble(&mem_secs);
+				mem_secs = atof (prop->value().c_str());
 			}
 			if ((prop = child_node->property ("discrete_io")) != 0) {
 				if (prop->value() == "yes") {
@@ -115,19 +123,19 @@ int LoopControl::SpawnConfig::set_state (const XMLNode& node)
 				}
 			}
 			if ((prop = child_node->property ("exec_name")) != 0) {
-				exec_name.Printf(wxT("%s"), prop->value().c_str());
+				exec_name = prop->value();
 			}
 			if ((prop = child_node->property ("jack_name")) != 0) {
-				jack_name.Printf(wxT("%s"), prop->value().c_str());
+				jack_name = prop->value();
 			}
 			if ((prop = child_node->property ("jack_serv_name")) != 0) {
-				jack_serv_name.Printf(wxT("%s"), prop->value().c_str());
+				jack_serv_name =  prop->value();
 			}
 			if ((prop = child_node->property ("midi_bind_path")) != 0) {
-			        midi_bind_path.Printf(wxT("%s"), prop->value().c_str());
+			        midi_bind_path = prop->value();
 			}
 			if ((prop = child_node->property ("session_path")) != 0) {
-			        session_path.Printf(wxT("%s"), prop->value().c_str());
+			        session_path = prop->value();
 			}
 			if ((prop = child_node->property ("force_spawn")) != 0) {
 				if (prop->value() == "yes") {
@@ -137,7 +145,16 @@ int LoopControl::SpawnConfig::set_state (const XMLNode& node)
 					force_spawn = false;
 				}
 			}
-
+			/*
+			if ((prop = child_node->property ("never_timeout")) != 0) {
+				if (prop->value() == "yes") {
+					never_timeout = true;
+				}
+				else {
+					never_timeout = false;
+				}
+			}
+			*/
 			break;
 		}
 	}
@@ -155,7 +172,9 @@ LoopControl::LoopControl (const wxString & rcdir)
 	_rcdir = rcdir;
 	_sentinel = true;
 	_lastchance = false;
-	
+	_we_spawned = false;
+	_engine_id = 0;
+
 	setup_param_map();
 	
 	_osc_server = lo_server_new(NULL, NULL);
@@ -165,6 +184,8 @@ LoopControl::LoopControl (const wxString & rcdir)
 	tmpstr = lo_server_get_url (_osc_server);
 	_our_url = tmpstr;
 	free (tmpstr);
+	
+	_our_port = lo_server_get_port(_osc_server);
 
 	cerr << "slgui: our URL is " << _our_url << endl;
 	
@@ -173,8 +194,10 @@ LoopControl::LoopControl (const wxString & rcdir)
 
 	// pingack expects: s:engine_url s:version i:loopcount
 	lo_server_add_method(_osc_server, "/pingack", "ssi", LoopControl::_pingack_handler, this);
+	lo_server_add_method(_osc_server, "/pingack", "ssii", LoopControl::_pingack_handler, this);
 
 	lo_server_add_method(_osc_server, "/alive_resp", "ssi", LoopControl::_alive_handler, this);
+	lo_server_add_method(_osc_server, "/alive_resp", "ssii", LoopControl::_alive_handler, this);
 
 	lo_server_add_method(_osc_server, "/error", "ss", LoopControl::_error_handler, this);
 	
@@ -257,20 +280,21 @@ LoopControl::terminate_traffic_thread ()
 void
 LoopControl::setup_param_map()
 {
-	state_map[LooperStateOff] = "off";
-	state_map[LooperStateWaitStart] = "waiting start";
-	state_map[LooperStateRecording] = "recording";
-	state_map[LooperStateWaitStop] = "waiting stop";
-	state_map[LooperStatePlaying] = "playing";
-	state_map[LooperStateOverdubbing] = "overdubbing";
-	state_map[LooperStateMultiplying] = "multiplying";
-	state_map[LooperStateInserting] = "inserting";
-	state_map[LooperStateReplacing] = "replacing";
-	state_map[LooperStateDelay] = "delay";
-	state_map[LooperStateMuted] = "muted";
-	state_map[LooperStateScratching] = "scratching";
-	state_map[LooperStateOneShot] = "one shot";
-	state_map[LooperStateSubstitute] = "substituting";
+	state_map[LooperStateOff] = wxT("off");
+	state_map[LooperStateWaitStart] = wxT("waiting start");
+	state_map[LooperStateRecording] = wxT("recording");
+	state_map[LooperStateWaitStop] = wxT("waiting stop");
+	state_map[LooperStatePlaying] = wxT("playing");
+	state_map[LooperStateOverdubbing] = wxT("overdubbing");
+	state_map[LooperStateMultiplying] = wxT("multiplying");
+	state_map[LooperStateInserting] = wxT("inserting");
+	state_map[LooperStateReplacing] = wxT("replacing");
+	state_map[LooperStateDelay] = wxT("delay");
+	state_map[LooperStateMuted] = wxT("muted");
+	state_map[LooperStatePaused] = wxT("paused");
+	state_map[LooperStateScratching] = wxT("scratching");
+	state_map[LooperStateOneShot] = wxT("one shot");
+	state_map[LooperStateSubstitute] = wxT("substituting");
 }
 
 
@@ -283,19 +307,27 @@ LoopControl::connect()
 	}
 
 	if (_spawn_config.host.empty()) {
-		_osc_addr = lo_address_new(NULL, wxString::Format(wxT("%ld"), _spawn_config.port).c_str());
+		_osc_addr = lo_address_new(NULL, (const char *) wxString::Format(wxT("%ld"), _spawn_config.port).ToAscii());
 	}
 	else {
-		_osc_addr = lo_address_new(_spawn_config.host.c_str(), wxString::Format(wxT("%ld"), _spawn_config.port).c_str());
+		_osc_addr = lo_address_new((const char *)_spawn_config.host.c_str(), (const char *) wxString::Format(wxT("%ld"), _spawn_config.port).ToAscii());
 	}
 	//cerr << "osc errstr: " << lo_address_errstr(_osc_addr) << endl;
 
+	// if the spawn_config host string is 127.0.0.1 or localhost, make our_url match it
+	if (_spawn_config.host == "127.0.0.1" || _spawn_config.host == "localhost") {
+		char tmpbuf[100];
+		snprintf (tmpbuf, sizeof(tmpbuf), "osc.udp://127.0.0.1:%d/", _our_port);
+		_our_url = tmpbuf;
+		cerr << "Changing our url to be : " << _our_url << endl;
+	}
+
 	_pingack = false;
 	
 	if (!_spawn_config.force_spawn) {
 		// send off a ping.  set a timer, if we don't have a response, we'll start our own locally
 		_waiting = 0;
-		lo_send(_osc_addr, "/ping", "ss", _our_url.c_str(), "/pingack");
+		lo_send(_osc_addr, "/ping", "ssi", _our_url.c_str(), "/pingack", 1);
 		//cerr << "sending ping" << endl;
 		_updatetimer->Start(700, true);
 	}
@@ -312,21 +344,52 @@ LoopControl::connect()
 	return true;
 }
 
+void
+LoopControl::register_global_updates(bool unreg)
+{
+	char buf[50];
+
+	if (!_osc_addr) return;
+	
+	if (unreg) {
+		snprintf(buf, sizeof(buf), "/unregister_update");
+
+	} else {
+		snprintf(buf, sizeof(buf), "/register_update");
+	}
+
+	// results will come back with instance = -2
+	lo_send(_osc_addr, buf, "sss", "tempo", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "sync_source", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "eighth_per_cycle", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "tap_tempo", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "wet", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "dry", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "input_gain", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "auto_disable_latency", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "output_midi_clock", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "use_midi_stop", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "use_midi_start", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "send_midi_start_on_trigger", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "smart_eighths", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "selected_loop_num", _our_url.c_str(), "/ctrl");
+	
+	if (unreg) {
+		lo_send(_osc_addr, "/unregister_auto_update", "siss", "in_peak_meter", 100, _our_url.c_str(), "/ctrl");
+		lo_send(_osc_addr, "/unregister_auto_update", "siss", "out_peak_meter", 100, _our_url.c_str(), "/ctrl");
+	}
+	else {
+		lo_send(_osc_addr, "/register_auto_update", "siss", "in_peak_meter", 100, _our_url.c_str(), "/ctrl");
+		lo_send(_osc_addr, "/register_auto_update", "siss", "out_peak_meter", 100, _our_url.c_str(), "/ctrl");
+	}
+}
+
 bool
 LoopControl::disconnect (bool killit)
 {
 
 	if (_osc_addr) {
-		lo_send(_osc_addr, "/unregister", "ss", _our_url.c_str(), "/pingack");
-		lo_send(_osc_addr, "/unregister_update", "sss", "tempo", _our_url.c_str(), "/ctrl");
-		lo_send(_osc_addr, "/unregister_update", "sss", "sync_source", _our_url.c_str(), "/ctrl");
-		lo_send(_osc_addr, "/unregister_update", "sss", "eighth_per_cycle", _our_url.c_str(), "/ctrl");
-		lo_send(_osc_addr, "/unregister_update", "sss", "tap_tempo", _our_url.c_str(), "/ctrl");
-		lo_send(_osc_addr, "/unregister_update", "sss", "wet", _our_url.c_str(), "/ctrl");
-		lo_send(_osc_addr, "/unregister_update", "sss", "dry", _our_url.c_str(), "/ctrl");
-
-		lo_send(_osc_addr, "/unregister_auto_update", "sss",  "in_peak_meter", _our_url.c_str(), "/ctrl");
-		lo_send(_osc_addr, "/unregister_auto_update", "sss",  "out_peak_meter", _our_url.c_str(), "/ctrl");
+		register_global_updates(true); // unregister
 		
 		if (killit) {
 			send_quit();
@@ -335,8 +398,9 @@ LoopControl::disconnect (bool killit)
 		lo_address_free (_osc_addr);
 		_osc_addr = 0;
 	}
-	_osc_url = wxT("");
-
+	_osc_url = "";
+	_we_spawned = false;
+	
 	Disconnected(); // emit
 	
 	return true;
@@ -417,12 +481,17 @@ LoopControl::is_engine_local()
 
 	// cerr << "osc: " << _osc_url << "  remhost: " << remhost << "  lochost: " << lochost << endl;
 	
-	if (remhost && lochost && (strncmp(remhost, lochost, 30) == 0)) {
+	if (_host == wxT("0.0.0.0") || // special case ridiculous mac-ness
+	    (remhost && lochost && (strncmp(remhost, lochost, 30) == 0))) {
 		ret = true;
 	}
 
-	free(remhost);
-	free(lochost);
+	if (remhost) {
+		free(remhost);
+	}
+	if (lochost) {
+		free(lochost);
+	}
 
 	return ret;
 }
@@ -453,17 +522,19 @@ LoopControl::pingtimer_expired()
 			}
 			else {
 				// last ditch effort with 127.0.0.1 all around
-				_osc_addr = lo_address_new("127.0.0.1", wxString::Format(wxT("%ld"), _spawn_config.port).c_str());
+				_osc_addr = lo_address_new("127.0.0.1", (const char *) wxString::Format(wxT("%ld"), _spawn_config.port).ToAscii());
 
+				char tmpbuf[100];
 				int sport = lo_server_get_port (_osc_server);
-				_our_url = wxString::Format(wxT("osc.udp://127.0.0.1:%d/"), sport); 
+				snprintf (tmpbuf, sizeof(tmpbuf), "osc.udp://127.0.0.1:%d/", sport);
+				_our_url = tmpbuf;
 
 				//cerr << "last chance effort: with oururl " << _our_url << endl;
 				
 				// send off a ping.  
 				_pingack = false;
 				_waiting = 60;
-				lo_send(_osc_addr, "/ping", "ss", _our_url.c_str(), "/pingack");
+				lo_send(_osc_addr, "/ping", "ssi", _our_url.c_str(), "/pingack", 1);
 				_lastchance = 1;
 				_updatetimer->Start(100, true);
 			}
@@ -483,7 +554,8 @@ LoopControl::pingtimer_expired()
 			_updatetimer->Start(100, true);
 			_waiting = 1;
 		}
-		else {
+		else if (!_spawn_config.never_timeout)
+		{
 			//cerr << "execute failed" << endl;
 			if (_osc_addr) {
 				lo_address_free(_osc_addr);
@@ -501,54 +573,76 @@ LoopControl::pingtimer_expired()
 bool LoopControl::spawn_looper()
 {
 	// use wxExecute
-	wxString cmdstr = _spawn_config.exec_name;
-
-	if (cmdstr.empty()) {
-		cmdstr = wxT("sooperlooper"); // always force something
-	}
+	char tmpbuf[300];
+	string cmdstr = _spawn_config.exec_name;
 	
-	cmdstr += wxString::Format(wxT(" -q -U %s -p %ld -l %ld -c %ld -t %d"),
-				   _our_url.c_str(),
-				   _spawn_config.port,
-				   _spawn_config.num_loops,
-				   _spawn_config.num_channels,
-				   (int) _spawn_config.mem_secs
-				   );
+	if (cmdstr.empty()) {
+		cmdstr = "sooperlooper"; // always force something
+	}
 
+	snprintf(tmpbuf, sizeof(tmpbuf), " -q -U %s -p %ld -l %ld -c %ld -t %d",
+		 _our_url.c_str(),
+		 _spawn_config.port,
+		 _spawn_config.num_loops,
+		 _spawn_config.num_channels,
+		 (int) _spawn_config.mem_secs
+		);
+	
+	cmdstr += tmpbuf;
+	
 	if (!_spawn_config.midi_bind_path.empty()) {
-		cmdstr += wxString::Format(wxT(" -m \"%s\""), _spawn_config.midi_bind_path.c_str());
+		snprintf(tmpbuf, sizeof(tmpbuf), " -m \"%s\"", _spawn_config.midi_bind_path.c_str());	
+		cmdstr += tmpbuf;
 	}
 	else {
 		// try default
-		wxString defpath = (_rcdir + wxFileName::GetPathSeparator() + wxT("default_midi.slb"));
-		cmdstr += wxString::Format(wxT(" -m \"%s\""), defpath.c_str());
+		//wxString defpath = (_rcdir + wxFileName::GetPathSeparator() + wxT("default_midi.slb"));
+		//cmdstr += wxString::Format(wxT(" -m \"%s\""), defpath.c_str());
+
+		string defpath = string(_rcdir.fn_str()) + "/" + "default_midi.slb";
+		snprintf(tmpbuf, sizeof(tmpbuf), " -m \"%s\"", defpath.c_str());	
+		cmdstr += tmpbuf;
 	}
 	
 	if (!_spawn_config.jack_name.empty()) {
-		cmdstr += wxString::Format(wxT(" -j \"%s\""), _spawn_config.jack_name.c_str());
+		//cmdstr += wxString::Format(wxT(" -j \"%s\""), _spawn_config.jack_name.c_str());
+		snprintf(tmpbuf, sizeof(tmpbuf), " -j \"%s\"", _spawn_config.jack_name.c_str());	
+		cmdstr += tmpbuf;
 	}
 
 	if (!_spawn_config.jack_serv_name.empty()) {
-		cmdstr += wxString::Format(wxT(" -S \"%s\""), _spawn_config.jack_serv_name.c_str());
+		//cmdstr += wxString::Format(wxT(" -S \"%s\""), _spawn_config.jack_serv_name.c_str());
+		snprintf(tmpbuf, sizeof(tmpbuf), " -S \"%s\"", _spawn_config.jack_serv_name.c_str());	
+		cmdstr += tmpbuf;
 	}
 
 	if (!_spawn_config.discrete_io) {
-		cmdstr += wxString::Format(wxT(" -D \"no\""));
+		//cmdstr += wxString::Format(wxT(" -D \"no\""));
+		snprintf(tmpbuf, sizeof(tmpbuf), " -D no");	
+		cmdstr += tmpbuf;
 	}
 	
 	if (!_spawn_config.session_path.empty()) {
-		cmdstr += wxString::Format(wxT(" -L \"%s\""), _spawn_config.session_path.c_str());
+		//cmdstr += wxString::Format(wxT(" -L \"%s\""), _spawn_config.session_path.c_str());
+		cerr << "session path not empty when spawning: " << _spawn_config.session_path << endl;
+		snprintf(tmpbuf, sizeof(tmpbuf), " -L \"%s\"", _spawn_config.session_path.c_str());	
+		cmdstr += tmpbuf;
 	}
 
 
-#ifdef DEBUG
-	cerr << "execing: '" << cmdstr << "'" << endl;
-#endif
-	_engine_pid = wxExecute(cmdstr, wxEXEC_ASYNC|wxEXEC_MAKE_GROUP_LEADER);
+//#ifdef DEBUG
+	cerr << "execing: '" << cmdstr.c_str() << "'" << endl << endl;
+//#endif
+	
+	_engine_pid = wxExecute(wxString::FromAscii(cmdstr.c_str()), wxEXEC_ASYNC|wxEXEC_MAKE_GROUP_LEADER);
 
 #ifdef DEBUG
 	cerr << "pid is " << _engine_pid << endl;
-#endif	
+#endif
+	if (_engine_pid > 0) {
+		_we_spawned = true;
+	}
+	
 	return _engine_pid > 0;
 }
 
@@ -565,32 +659,50 @@ LoopControl::pingack_handler(const char *path, const char *types, lo_arg **argv,
 {
 	// s:hosturl  s:version  i:loopcount
 
-	wxString hosturl(&argv[0]->s);
-	wxString version(&argv[1]->s);
+	string hosturl(&argv[0]->s);
+	string version(&argv[1]->s);
 	int loopcount = argv[2]->i;
+	int uid = 0;
+	char tmpbuf[100];
+	
+	if (argc > 3) {
+		uid = argv[3]->i;
+	}
 
-	cerr << "slgui: remote looper is at " << hosturl << " version=" << version << "   loopcount=" << loopcount << endl;
+	cerr << "slgui: remote looper is at " << hosturl << " version=" << version << "   loopcount=" << loopcount << "  id=" << uid << endl;
 
 	char * remport = lo_url_get_port(hosturl.c_str());
-	wxString tmpstr(remport);
+	wxString tmpstr = wxString::FromAscii(remport);
 	tmpstr.ToLong(&_spawn_config.port);
 	free(remport);
 	
 	if (_lastchance) {
 		// force a 127.0.0.1 hostname if this was on our last chance
 		//cerr << "forced a 127" << endl;
-		_osc_url = wxString::Format(wxT("osc.udp://127.0.0.1:%ld/"), _spawn_config.port);
+		snprintf(tmpbuf, sizeof(tmpbuf), "osc.udp://127.0.0.1:%ld/", _spawn_config.port);
+		_osc_url = tmpbuf;
 	}
 	else {
 		_osc_url = hosturl;
 	}
 	
 	char * remhost = lo_url_get_hostname(_osc_url.c_str());
-	_host = remhost;
+	if (_spawn_config.host.empty() && !_lastchance) {
+		// only if the spawn config host was empty do we use what is returned
+		_host = wxString::FromAscii(remhost);
+		_spawn_config.host = remhost;
+	}
+	else {
+		// use the one given
+		_host = wxString::FromAscii(_spawn_config.host.c_str());
+		snprintf(tmpbuf, sizeof(tmpbuf), "osc.udp://%s:%ld/", _spawn_config.host.c_str(), _spawn_config.port);
+		_osc_url = tmpbuf;
+		cerr << "  but treating the engine URL as " << _osc_url << endl;
+	}
+
 	free(remhost);
 
 
-	_spawn_config.host = _host;
 
 	_port = (int) _spawn_config.port;
 
@@ -599,23 +711,20 @@ LoopControl::pingack_handler(const char *path, const char *types, lo_arg **argv,
 	}
 	_osc_addr = lo_address_new_from_url (hosturl.c_str());
 
+	if (_engine_id != 0 && _engine_id != uid) {
+		cerr << "new engine ID pingacked us!, re-registering" << endl;
+		_pingack = false;
+	}
+	_engine_id = uid;
+
 	if (!_pingack) {
 		// register future configs with it once
 		lo_send(_osc_addr, "/register", "ss", _our_url.c_str(), "/pingack");
 
-		// results will come back with instance = -2
-		lo_send(_osc_addr, "/register_update", "sss", "tempo", _our_url.c_str(), "/ctrl");
-		lo_send(_osc_addr, "/register_update", "sss", "sync_source", _our_url.c_str(), "/ctrl");
-		lo_send(_osc_addr, "/register_update", "sss", "eighth_per_cycle", _our_url.c_str(), "/ctrl");
-		lo_send(_osc_addr, "/register_update", "sss", "tap_tempo", _our_url.c_str(), "/ctrl");
-		lo_send(_osc_addr, "/register_update", "sss", "wet", _our_url.c_str(), "/ctrl");
-		lo_send(_osc_addr, "/register_update", "sss", "dry", _our_url.c_str(), "/ctrl");
+		register_global_updates();		
+
+		request_all_midi_bindings();
 
-		lo_send(_osc_addr, "/register_auto_update", "siss", "in_peak_meter", 100, _our_url.c_str(), "/ctrl");
-		lo_send(_osc_addr, "/register_auto_update", "siss", "out_peak_meter", 100, _our_url.c_str(), "/ctrl");
-		
-		lo_send(_osc_addr, "/get_all_midi_bindings", "ss", _our_url.c_str(), "/recv_midi_bindings");
-		
 		_pingack = true;
 	}
 
@@ -636,7 +745,14 @@ LoopControl::_alive_handler(const char *path, const char *types, lo_arg **argv,
 int
 LoopControl::alive_handler(const char *path, const char *types, lo_arg **argv, int argc, void *data)
 {
-	// s:hosturl  s:version  i:loopcount
+	// s:hosturl  s:version  i:loopcount [i:id]
+	if (argc > 3) {
+		int uid = argv[3]->i;
+		if (uid != _engine_id) {
+			cerr << "engine changed on us, redoing connections" << endl;
+			return pingack_handler(path, types, argv, argc, data);
+		}
+	}
 
 	IsAlive(); // emit
 	return 0;
@@ -678,9 +794,11 @@ LoopControl::control_handler(const char *path, const char *types, lo_arg **argv,
 	// loop instance is 1st arg, 2nd is ctrl string, 3rd is float value
 
 	int  index = argv[0]->i;
-	wxString ctrl(&argv[1]->s);
+	wxString ctrl = wxString::FromAscii(&argv[1]->s);
 	float val  = argv[2]->f;
 
+
+	// cerr << "got " << ctrl.ToAscii() << " = " << val << "  index=" << index << endl;
 	
 	if (index == -2)
 	{
@@ -712,7 +830,6 @@ LoopControl::control_handler(const char *path, const char *types, lo_arg **argv,
 	
 	_params_val_map[index][ctrl] = val;
 
-	//cerr << "got " << ctrl << " = " << val << "  index=" << index << endl;
 	
 	return 0;
 }
@@ -757,6 +874,28 @@ LoopControl::midi_binding_handler(const char *path, const char *types, lo_arg **
 	return 0;
 }
 
+void
+LoopControl::request_control_value (int index,  wxString ctrl)
+{
+	if (!_osc_addr) return;
+	char buf[20];
+
+	snprintf(buf, sizeof(buf), "/sl/%d/get", index);
+	
+	lo_send(_osc_addr, buf, "sss", (const char *) ctrl.ToAscii(), _our_url.c_str(), "/ctrl");
+}
+
+void
+LoopControl::request_global_control_value (wxString ctrl)
+{
+	if (!_osc_addr) return;
+	char buf[20];
+
+	snprintf(buf, sizeof(buf), "/get");
+	
+	lo_send(_osc_addr, buf, "sss", (const char *) ctrl.ToAscii(), _our_url.c_str(), "/ctrl");
+}
+
 void
 LoopControl::request_global_values()
 {
@@ -772,6 +911,14 @@ LoopControl::request_global_values()
 	//lo_send(_osc_addr, buf, "sss", "tap_tempo", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "dry", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "wet", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "input_gain", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "auto_disable_latency", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "smart_eighths", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "output_midi_clock", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "use_midi_stop", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "use_midi_start", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "send_midi_start_on_trigger", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "selected_loop_num", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "in_peak_meter", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "out_peak_meter", _our_url.c_str(), "/ctrl");
 
@@ -817,6 +964,7 @@ LoopControl::request_all_values(int index)
 	lo_send(_osc_addr, buf, "sss", "use_feedback_play", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "dry", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "wet", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "input_gain", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "rate", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "scratch_pos", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "delay_trigger", _our_url.c_str(), "/ctrl");
@@ -831,6 +979,17 @@ LoopControl::request_all_values(int index)
 	lo_send(_osc_addr, buf, "sss", "use_common_ins", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "has_discrete_io", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "relative_sync", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "input_latency", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "output_latency", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "trigger_latency", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "autoset_latency", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "mute_quantized", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "overdub_quantized", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "replace_quantized", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "round_integer_tempo", _our_url.c_str(), "/ctrl");
+	//lo_send(_osc_addr, buf, "sss", "pitch_shift", _our_url.c_str(), "/ctrl");
+	//lo_send(_osc_addr, buf, "sss", "stretch_ratio", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "tempo_stretch", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "pan_1", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "pan_2", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "pan_3", _our_url.c_str(), "/ctrl");
@@ -908,7 +1067,7 @@ LoopControl::cancel_midi_learn()
 void
 LoopControl::load_midi_bindings(const wxString & filename, bool append)
 {
-	lo_send(_osc_addr, "/load_midi_bindings", "ss", filename.c_str(), append ? "add": "");
+	lo_send(_osc_addr, "/load_midi_bindings", "ss", (const char *) filename.ToAscii(), append ? "add": "");
 
 	request_all_midi_bindings();
 }
@@ -918,7 +1077,7 @@ LoopControl::save_midi_bindings(const wxString & filename)
 {
 	if (!_osc_addr) return;
 
-	lo_send(_osc_addr, "/save_midi_bindings", "ss", filename.c_str(), "");
+	lo_send(_osc_addr, "/save_midi_bindings", "ss", (const char *) filename.ToAscii(), "");
 }
 
 void
@@ -940,7 +1099,9 @@ LoopControl::register_auto_updates(int index, bool unreg)
 		lo_send(_osc_addr, buf, "sss", "rate_output", _our_url.c_str(), "/ctrl");
 		lo_send(_osc_addr, buf, "sss", "in_peak_meter", _our_url.c_str(), "/ctrl");
 		lo_send(_osc_addr, buf, "sss", "out_peak_meter", _our_url.c_str(), "/ctrl");
-
+		lo_send(_osc_addr, buf, "sss", "is_soloed", _our_url.c_str(), "/ctrl");
+		lo_send(_osc_addr, buf, "sss", "stretch_ratio", _our_url.c_str(), "/ctrl");
+		lo_send(_osc_addr, buf, "sss", "pitch_shift", _our_url.c_str(), "/ctrl");
 	} else {
 		snprintf(buf, sizeof(buf), "/sl/%d/register_auto_update", index);
 		// send request for auto updates
@@ -955,6 +1116,9 @@ LoopControl::register_auto_updates(int index, bool unreg)
 		lo_send(_osc_addr, buf, "siss", "rate_output",100, _our_url.c_str(), "/ctrl");
 		lo_send(_osc_addr, buf, "siss", "in_peak_meter", 100, _our_url.c_str(), "/ctrl");
 		lo_send(_osc_addr, buf, "siss", "out_peak_meter", 100, _our_url.c_str(), "/ctrl");
+		lo_send(_osc_addr, buf, "siss",  "is_soloed", 100, _our_url.c_str(), "/ctrl");
+		lo_send(_osc_addr, buf, "siss", "stretch_ratio", 100, _our_url.c_str(), "/ctrl");
+		lo_send(_osc_addr, buf, "siss", "pitch_shift", 100, _our_url.c_str(), "/ctrl");
 	}
 
 	
@@ -965,7 +1129,7 @@ void
 LoopControl::register_input_controls(int index, bool unreg)
 {
 	if (!_osc_addr) return;
-	char buf[30];
+	char buf[50];
 
 	if ((int)_params_val_map.size() > index) {
 		_params_val_map[index].clear();
@@ -985,6 +1149,7 @@ LoopControl::register_input_controls(int index, bool unreg)
 	lo_send(_osc_addr, buf, "sss", "use_feedback_play", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "dry", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "wet", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "input_gain", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "rate", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "scratch_pos", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "delay_trigger", _our_url.c_str(), "/ctrl");
@@ -998,6 +1163,17 @@ LoopControl::register_input_controls(int index, bool unreg)
 	lo_send(_osc_addr, buf, "sss", "use_common_ins", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "use_common_outs", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "relative_sync", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "input_latency", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "output_latency", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "trigger_latency", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "autoset_latency", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "mute_quantized", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "overdub_quantized", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "replace_quantized", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "round_integer_tempo", _our_url.c_str(), "/ctrl");
+	//lo_send(_osc_addr, buf, "sss", "pitch_shift", _our_url.c_str(), "/ctrl");
+	//lo_send(_osc_addr, buf, "sss", "stretch_ratio", _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", "tempo_stretch", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "pan_1", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "pan_2", _our_url.c_str(), "/ctrl");
 	lo_send(_osc_addr, buf, "sss", "pan_3", _our_url.c_str(), "/ctrl");
@@ -1024,7 +1200,7 @@ LoopControl::register_control (int index, wxString ctrl, bool unreg)
 	}
 	
 	// send request for updates
-	lo_send(_osc_addr, buf, "sss", ctrl.c_str(), _our_url.c_str(), "/ctrl");
+	lo_send(_osc_addr, buf, "sss", (const char *) ctrl.ToAscii(), _our_url.c_str(), "/ctrl");
 
 }
 
@@ -1033,17 +1209,17 @@ LoopControl::register_auto_update(int index, wxString ctrl, bool unreg)
 {
 	if (!_osc_addr) return;
 
-	char buf[30];
+	char buf[50];
 
 	if (unreg) {
 		snprintf(buf, sizeof(buf), "/sl/%d/unregister_auto_update", index);
 
-		lo_send(_osc_addr, buf, "sss", ctrl.c_str(), _our_url.c_str(), "/ctrl");
+		lo_send(_osc_addr, buf, "sss", (const char *) ctrl.ToAscii(), _our_url.c_str(), "/ctrl");
 
 	} else {
 		snprintf(buf, sizeof(buf), "/sl/%d/register_auto_update", index);
 
-		lo_send(_osc_addr, buf, "siss", ctrl.c_str(), 100, _our_url.c_str(), "/ctrl");
+		lo_send(_osc_addr, buf, "siss", (const char *) ctrl.ToAscii(), 100, _our_url.c_str(), "/ctrl");
 	}
 	
 }
@@ -1053,11 +1229,11 @@ bool
 LoopControl::post_down_event(int index, wxString cmd)
 {
 	if (!_osc_addr) return false;
-	char buf[30];
+	char buf[50];
 
 	snprintf(buf, sizeof(buf), "/sl/%d/down", index);
 	
-	if (lo_send(_osc_addr, buf, "s", cmd.c_str()) == -1) {
+	if (lo_send(_osc_addr, buf, "s", (const char *)cmd.ToAscii()) == -1) {
 		return false;
 	}
 
@@ -1068,7 +1244,7 @@ bool
 LoopControl::post_up_event(int index, wxString cmd, bool force)
 {
 	if (!_osc_addr) return false;
-	char buf[30];
+	char buf[50];
 
 	if (force) {
 		snprintf(buf, sizeof(buf), "/sl/%d/upforce", index);
@@ -1077,7 +1253,7 @@ LoopControl::post_up_event(int index, wxString cmd, bool force)
 		snprintf(buf, sizeof(buf), "/sl/%d/up", index);
 	}
 	
-	if (lo_send(_osc_addr, buf, "s", cmd.c_str()) == -1) {
+	if (lo_send(_osc_addr, buf, "s", (const char *) cmd.ToAscii()) == -1) {
 		return false;
 	}
 
@@ -1088,7 +1264,7 @@ bool
 LoopControl::post_ctrl_change (int index, wxString ctrl, float val)
 {
 	if (!_osc_addr) return false;
-	char buf[30];
+	char buf[50];
 
 	// go ahead and update our local copy
 	if (index >= 0 && index < (int) _params_val_map.size()) {
@@ -1097,7 +1273,7 @@ LoopControl::post_ctrl_change (int index, wxString ctrl, float val)
 	
 	snprintf(buf, sizeof(buf), "/sl/%d/set", index);
 
-	if (lo_send(_osc_addr, buf, "sf", ctrl.c_str(), val) == -1) {
+	if (lo_send(_osc_addr, buf, "sf", (const char *) ctrl.ToAscii(), val) == -1) {
 		return false;
 	}
 
@@ -1112,7 +1288,7 @@ LoopControl::post_global_ctrl_change (wxString ctrl, float val)
 	// go ahead and update our local copy
 	_global_val_map[ctrl] = val;
 	
-	if (lo_send(_osc_addr, "/set", "sf", ctrl.c_str(), val) == -1) {
+	if (lo_send(_osc_addr, "/set", "sf", (const char *) ctrl.ToAscii(), val) == -1) {
 		return false;
 	}
 
@@ -1123,12 +1299,14 @@ bool
 LoopControl::post_save_loop(int index, wxString fname, wxString format, wxString endian)
 {
 	if (!_osc_addr) return false;
-	char buf[30];
+	char buf[50];
 
 	snprintf(buf, sizeof(buf), "/sl/%d/save_loop", index);
 
 	// send request for updates
-	if (lo_send(_osc_addr, buf, "sssss", fname.c_str(), format.c_str(), endian.c_str(), _our_url.c_str(), "/error") == -1) {
+	if (lo_send(_osc_addr, buf, "sssss", (const char *) fname.ToAscii(),
+		    (const char *) format.ToAscii(), (const char *) endian.ToAscii(),
+		    (const char *) _our_url.c_str(), "/error") == -1) {
 		return false;
 	}
 
@@ -1144,7 +1322,7 @@ LoopControl::post_load_loop(int index, wxString fname)
 	snprintf(buf, sizeof(buf), "/sl/%d/load_loop", index);
 
 	// send request for updates
-	if (lo_send(_osc_addr, buf, "sss", fname.c_str(), _our_url.c_str(), "/error") == -1) {
+	if (lo_send(_osc_addr, buf, "sss", (const char *) fname.ToAscii(), _our_url.c_str(), "/error") == -1) {
 		return false;
 	}
 
@@ -1160,7 +1338,7 @@ LoopControl::post_load_session(wxString fname)
 	snprintf(buf, sizeof(buf), "/load_session");
 
 	// send request for updates
-	if (lo_send(_osc_addr, buf, "sss", fname.c_str(), _our_url.c_str(), "/error") == -1) {
+	if (lo_send(_osc_addr, buf, "sss", (const char *) fname.ToAscii(), _our_url.c_str(), "/error") == -1) {
 		return false;
 	}
 
@@ -1168,16 +1346,23 @@ LoopControl::post_load_session(wxString fname)
 }
 
 bool
-LoopControl::post_save_session(wxString fname)
+LoopControl::post_save_session(wxString fname, bool write_audio)
 {
 	if (!_osc_addr) return false;
 	char buf[30];
 
 	snprintf(buf, sizeof(buf), "/save_session");
 
-	// send request for updates
-	if (lo_send(_osc_addr, buf, "sss", fname.c_str(), _our_url.c_str(), "/error") == -1) {
-		return false;
+
+	if (write_audio) {
+		if (lo_send(_osc_addr, buf, "sssi", (const char *) fname.ToAscii(), _our_url.c_str(), "/error", write_audio) == -1) {
+			return false;
+		}
+	}
+	else {
+		if (lo_send(_osc_addr, buf, "sss", (const char *) fname.ToAscii(), _our_url.c_str(), "/error") == -1) {
+			return false;
+		}
 	}
 
 	return true;
@@ -1195,7 +1380,7 @@ void
 LoopControl::send_alive_ping()
 {
 	if (!_osc_addr) return;
-	lo_send(_osc_addr, "/ping", "ss", _our_url.c_str(), "/alive_resp");
+	lo_send(_osc_addr, "/ping", "ssi", _our_url.c_str(), "/alive_resp", 1);
 }
 
 void
@@ -1285,13 +1470,13 @@ LoopControl::get_state (int index, LooperState & state, wxString & statestr)
 
 	if (index >= 0 && index < (int) _params_val_map.size())
 	{
-		ControlValMap::iterator iter = _params_val_map[index].find ("state");
+		ControlValMap::iterator iter = _params_val_map[index].find (wxT("state"));
 
 		if (iter != _params_val_map[index].end()) {
-			state = (LooperState) (*iter).second;
+			state = (LooperState) (int) (*iter).second;
 			statestr = state_map[state];
 			// set updated to false
-			_updated[index]["state"] = false;
+			_updated[index][wxT("state")] = false;
 			ret = true;
 		}
 	}
@@ -1306,13 +1491,13 @@ LoopControl::get_next_state (int index, LooperState & state, wxString & statestr
 
 	if (index >= 0 && index < (int) _params_val_map.size())
 	{
-		ControlValMap::iterator iter = _params_val_map[index].find ("next_state");
+		ControlValMap::iterator iter = _params_val_map[index].find (wxT("next_state"));
 
 		if (iter != _params_val_map[index].end()) {
-			state = (LooperState) (*iter).second;
+			state = (LooperState) (int) (*iter).second;
 			statestr = state_map[state];
 			// set updated to false
-			_updated[index]["next_state"] = false;
+			_updated[index][wxT("next_state")] = false;
 			ret = true;
 		}
 	}
