@@ -25,8 +25,7 @@
 #include <cstring>
 #include <cmath>
 
-#include "gui_app.hpp"
-#include "gui_frame.hpp"
+#include "main_panel.hpp"
 #include "keyboard_target.hpp"
 #include "looper_panel.hpp"
 #include "pix_button.hpp"
@@ -35,6 +34,7 @@
 #include "slider_bar.hpp"
 #include "choice_box.hpp"
 #include "check_box.hpp"
+#include "spin_box.hpp"
 
 #include "pixmap_includes.hpp"
 
@@ -57,6 +57,8 @@ enum {
 	ID_ReverseButton,
 	ID_SubstituteButton,
 	ID_MuteButton,
+	ID_PauseButton,
+	ID_SoloButton,
 	ID_RateButton,
 	ID_ScratchButton,
 	ID_LoadButton,
@@ -73,12 +75,20 @@ enum {
 	ID_WetControl,
 	ID_ScratchControl,
 	ID_RateControl,
+	ID_StretchControl,
+	ID_PitchControl,
+	ID_InputGainControl,
 
+	ID_InputLatency,
+	ID_OutputLatency,
+	ID_TriggerLatency,
+	
 	ID_QuantizeCheck,
 	ID_QuantizeChoice,
 	ID_RoundCheck,
 	ID_SyncCheck,
 	ID_UseFeedbackPlayCheck,
+	ID_TempoStretchCheck,
 	ID_PlaySyncCheck,
 	ID_UseMainInCheck,
 	ID_Panner,
@@ -98,9 +108,10 @@ BEGIN_EVENT_TABLE(LooperPanel, wxPanel)
 	
 END_EVENT_TABLE()
 
-LooperPanel::LooperPanel(LoopControl * control, wxWindow * parent, wxWindowID id, const wxPoint& pos, const wxSize& size)
+	LooperPanel::LooperPanel(MainPanel * mainpan, LoopControl * control, wxWindow * parent, wxWindowID id, const wxPoint& pos, const wxSize& size)
 	: wxPanel(parent, id, pos, size), _loop_control(control), _index(0), _last_state(LooperStateUnknown), _tap_val(1.0f)
 {
+	_mainpanel = mainpan;
 	_learning = false;
 	_scratch_pressed = false;
 	_last_state = LooperStateUnknown;
@@ -175,21 +186,29 @@ LooperPanel::init()
 	wxFont sliderFont = *wxSMALL_FONT;
 	//cerr << "looper frame small: " << sliderFont.GetPointSize() << endl;
 	
+	wxBoxSizer * inthresh_sizer = new wxBoxSizer(wxHORIZONTAL);
 
+	_in_gain_control = slider = new SliderBar(this, ID_InputGainControl, 0.0f, 1.0f, 0.0f);
+	slider->set_units(wxT(""));
+	slider->set_label(wxT("in gain"));
+	slider->set_show_indicator_bar (false);
+	slider->set_scale_mode(SliderBar::ZeroGainMode);
+	slider->SetFont(sliderFont);
+	slider->value_changed.connect (bind (slot (*this, &LooperPanel::slider_events), (int) slider->GetId()));
+	slider->bind_request.connect (bind (slot (*this, &LooperPanel::control_bind_events), (int) slider->GetId()));
+	inthresh_sizer->Add (slider, 1, wxALL|wxEXPAND, 0);
+	
 	_thresh_control = slider = new SliderBar(this, ID_ThreshControl, 0.0f, 1.0f, 0.0f);
-	slider->set_units(wxT("dB"));
+	slider->set_units(wxT(""));
 	slider->set_label(wxT("thresh"));
 	slider->set_show_indicator_bar (true);
 	slider->set_scale_mode(SliderBar::ZeroGainMode);
 	slider->SetFont(sliderFont);
 	slider->value_changed.connect (bind (slot (*this, &LooperPanel::slider_events), (int) slider->GetId()));
 	slider->bind_request.connect (bind (slot (*this, &LooperPanel::control_bind_events), (int) slider->GetId()));
-	_maininsizer->Add (slider, 1, wxALL|wxEXPAND, 0);
-
-	// mainin check added later
-	_use_main_in_check = 0;
+	inthresh_sizer->Add (slider, 1, wxLEFT|wxEXPAND, 3);
 	
-	colsizer->Add (_maininsizer, 1, wxEXPAND|wxTOP|wxLEFT, 5);
+	colsizer->Add (inthresh_sizer, 1, wxEXPAND|wxTOP|wxLEFT, 5);
 
 	_feedback_control = slider = new SliderBar(this, ID_FeedbackControl, 0.0f, 100.0f, 100.0f);
 	slider->set_units(wxT("%"));
@@ -197,8 +216,16 @@ LooperPanel::init()
 	slider->SetFont(sliderFont);
 	slider->value_changed.connect (bind (slot (*this, &LooperPanel::slider_events), (int) slider->GetId()));
 	slider->bind_request.connect (bind (slot (*this, &LooperPanel::control_bind_events), (int) slider->GetId()));
-	colsizer->Add (slider, 1, wxEXPAND|wxTOP|wxLEFT, 5);
 
+	_maininsizer->Add (slider, 1, wxALL|wxEXPAND, 0);
+
+	// mainin check added later
+	_use_main_in_check = 0;
+	
+	colsizer->Add (_maininsizer, 1, wxEXPAND|wxTOP|wxLEFT, 5);
+
+
+	
 	//colsizer->Add (20, 5, 0, wxEXPAND);
 	rowsizer = new wxBoxSizer(wxHORIZONTAL);
  	rowsizer->Add (_replace_button, 0, wxTOP|wxLEFT, 5);
@@ -245,10 +272,10 @@ LooperPanel::init()
 	colsizer->Add (_toppansizer, 1, wxEXPAND|wxTOP|wxLEFT, 4);
 
 	_botpansizer = new wxBoxSizer(wxHORIZONTAL);
-	
+
 	_wet_control = slider = new SliderBar(this, ID_WetControl, 0.0f, 1.0f, 1.0f);
 	slider->set_units(wxT("dB"));
-	slider->set_label(wxT("wet"));
+	slider->set_label(wxT("out"));
 	slider->set_show_indicator_bar (true);
 	slider->set_scale_mode(SliderBar::ZeroGainMode);
 	slider->SetFont(sliderFont);
@@ -256,6 +283,26 @@ LooperPanel::init()
 	slider->bind_request.connect (bind (slot (*this, &LooperPanel::control_bind_events), (int) slider->GetId()));
 	_botpansizer->Add (slider, 1, wxEXPAND, 0);
 
+	/*
+	_outlatency_spin =  new SpinBox(this, ID_OutputLatency, 0.0f, 32768.0f, 0.0f, true, wxDefaultPosition, wxSize(65, 20));
+	_outlatency_spin->set_units(wxT(""));
+	_outlatency_spin->set_label(wxT("o.lat"));
+	_outlatency_spin->set_snap_mode (SpinBox::IntegerSnap);
+	_outlatency_spin->set_allow_outside_bounds(true);
+	_outlatency_spin->value_changed.connect (bind (slot (*this, &LooperPanel::slider_events), (int) _outlatency_spin->GetId()));
+	_outlatency_spin->SetFont(sliderFont);
+	_botpansizer->Add (_outlatency_spin, 0, wxALL, 0);
+
+	_inlatency_spin =  new SpinBox(this, ID_InputLatency, 0.0f, 32768.0f, 0.0f, true, wxDefaultPosition, wxSize(65, 20));
+	_inlatency_spin->set_units(wxT(""));
+	_inlatency_spin->set_label(wxT("i.lat"));
+	_inlatency_spin->set_snap_mode (SpinBox::IntegerSnap);
+	_inlatency_spin->set_allow_outside_bounds(true);
+	_inlatency_spin->value_changed.connect (bind (slot (*this, &LooperPanel::slider_events), (int) _inlatency_spin->GetId()));
+	_inlatency_spin->SetFont(sliderFont);
+	_botpansizer->Add (_inlatency_spin, 0, wxALL, 0);
+	*/
+	
 	colsizer->Add (_botpansizer, 1, wxEXPAND|wxTOP|wxLEFT, 4);
 	
 	
@@ -317,6 +364,14 @@ LooperPanel::init()
 	_play_feed_check->value_changed.connect (bind (slot (*this, &LooperPanel::check_events), wxT("use_feedback_play")));
 	_play_feed_check->bind_request.connect (bind (slot (*this, &LooperPanel::control_bind_events), (int) _play_feed_check->GetId()));
 	lilrowsizer->Add (_play_feed_check, 1, wxLEFT, 3);
+
+	_tempo_stretch_check = new CheckBox(this, ID_TempoStretchCheck, wxT("t. stretch"), true, wxDefaultPosition, wxSize(55, 18));
+	_tempo_stretch_check->SetFont(sliderFont);
+	_tempo_stretch_check->SetToolTip(wxT("enable automatic timestretch when tempo changes"));
+	_tempo_stretch_check->value_changed.connect (bind (slot (*this, &LooperPanel::check_events), wxT("tempo_stretch")));
+	_tempo_stretch_check->bind_request.connect (bind (slot (*this, &LooperPanel::control_bind_events), (int) _tempo_stretch_check->GetId()));
+	lilrowsizer->Add (_tempo_stretch_check, 1, wxLEFT, 3);
+
 	lilcolsizer->Add (lilrowsizer, 0, wxTOP|wxEXPAND, 0);
 	
 	rowsizer->Add(lilcolsizer, 1, wxTOP|wxLEFT, 3);
@@ -338,9 +393,14 @@ LooperPanel::init()
 	
 	rowsizer->Add(lilcolsizer, 0, wxTOP|wxLEFT, 3);
 	
+	lilcolsizer = new wxBoxSizer(wxVERTICAL);
 	
- 	rowsizer->Add (_mute_button, 0, wxTOP|wxLEFT, 5);
-	
+ 	lilcolsizer->Add (_mute_button, 0, wxTOP, 2);
+
+ 	lilcolsizer->Add (_solo_button, 0, wxTOP, 2);
+
+	rowsizer->Add(lilcolsizer, 0, wxTOP|wxLEFT, 3);	
+
 	colsizer->Add (rowsizer, 0, wxEXPAND);
 
 	colsizer->Add (20,-1, 1);
@@ -365,6 +425,22 @@ LooperPanel::init()
 	slider->value_changed.connect (bind (slot (*this, &LooperPanel::slider_events), (int) slider->GetId()));
 	slider->bind_request.connect (bind (slot (*this, &LooperPanel::control_bind_events), (int) slider->GetId()));
 	rowsizer->Add (slider, 1, wxEXPAND|wxTOP|wxLEFT, 3);
+
+	// pitch control
+	_pitch_control = slider = new SliderBar(this, ID_PitchControl, -12.0f, 12.0f, 0.0f);
+	slider->set_units(wxT(""));
+	slider->set_label(wxT("pitch"));
+	slider->set_style (SliderBar::CenterStyle);
+	slider->set_decimal_digits (1);
+	slider->set_snap_mode(SliderBar::IntegerSnap);
+	slider->SetFont(sliderFont);
+	slider->value_changed.connect (bind (slot (*this, &LooperPanel::slider_events), (int) slider->GetId()));
+	slider->bind_request.connect (bind (slot (*this, &LooperPanel::control_bind_events), (int) slider->GetId()));
+	rowsizer->Add (slider, 1, wxEXPAND|wxTOP|wxLEFT, 3);
+
+	// pause
+ 	rowsizer->Add (_pause_button, 0, wxTOP|wxLEFT, 3);
+
 	
 	colsizer->Add (rowsizer, 0, wxEXPAND);
 
@@ -385,6 +461,29 @@ LooperPanel::init()
 	slider->value_changed.connect (bind (slot (*this, &LooperPanel::slider_events), (int) slider->GetId()));
 	slider->bind_request.connect (bind (slot (*this, &LooperPanel::control_bind_events), (int) slider->GetId()));
 	rowsizer->Add (slider, 1, wxEXPAND|wxTOP|wxLEFT, 3);
+
+	// stretch control
+	_stretch_control = slider = new SliderBar(this, ID_StretchControl, 0.5f, 4.0f, 1.0f);
+	slider->set_units(wxT(""));
+	slider->set_label(wxT("stretch"));
+	slider->set_style (SliderBar::CenterStyle);
+	slider->set_decimal_digits (2);
+	slider->SetFont(sliderFont);
+	slider->value_changed.connect (bind (slot (*this, &LooperPanel::slider_events), (int) slider->GetId()));
+	slider->bind_request.connect (bind (slot (*this, &LooperPanel::control_bind_events), (int) slider->GetId()));
+	rowsizer->Add (slider, 1, wxEXPAND|wxTOP|wxLEFT, 3);
+
+
+	/*
+	_triglatency_spin =  new SpinBox(this, ID_TriggerLatency, 0.0f, 32768.0f, 0.0f, true, wxDefaultPosition, wxSize(65, 20));
+	_triglatency_spin->set_units(wxT(""));
+	_triglatency_spin->set_label(wxT("t.lat"));
+	_triglatency_spin->set_snap_mode (SpinBox::IntegerSnap);
+	_triglatency_spin->set_allow_outside_bounds(true);
+	_triglatency_spin->value_changed.connect (bind (slot (*this, &LooperPanel::slider_events), (int) _triglatency_spin->GetId()));
+	_triglatency_spin->SetFont(sliderFont);
+	rowsizer->Add (_triglatency_spin, 0, wxALL, 0);
+	*/
 	
 	colsizer->Add (rowsizer, 0, wxEXPAND|wxLEFT, 1);
 
@@ -429,7 +528,7 @@ LooperPanel::post_init()
 		// dry is only meaningful with discrete io
 		_dry_control = slider = new SliderBar(this, ID_DryControl, 0.0f, 1.0f, 1.0f);
 		slider->set_units(wxT("dB"));
-		slider->set_label(wxT("dry"));
+		slider->set_label(wxT("in mon"));
 		slider->set_scale_mode(SliderBar::ZeroGainMode);
 		slider->SetFont(sliderFont);
 		slider->value_changed.connect (bind (slot (*this, &LooperPanel::slider_events), (int) slider->GetId()));
@@ -444,11 +543,10 @@ LooperPanel::post_init()
 		_maininsizer->Add (_use_main_in_check, 0, wxALL|wxEXPAND|wxALIGN_CENTRE_VERTICAL ,0);
 		_maininsizer->Layout();
 
+		_feedback_control->set_label(wxT("feedb"));
+
 	}
 	else {
-		// no discrete io, thus the inputs will be common inputs
-		// and we can unregister for input peaks
-		_loop_control->register_auto_update (_index, wxT("in_peak_meter"), true);
 		_has_discrete_io = false;
 		_dry_control = 0;
 	}
@@ -490,6 +588,7 @@ LooperPanel::post_init()
 
 	_toppansizer->Layout();
 	_botpansizer->Layout();
+	GetSizer()->Layout();
 	Refresh(false);
 }
 
@@ -517,57 +616,65 @@ LooperPanel::set_index(int ind)
 void
 LooperPanel::bind_events()
 {
-	_undo_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString("undo")));
-	_undo_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString("undo")));
-	_undo_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString("undo")));
+	_undo_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString(wxT("undo"))));
+	_undo_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString(wxT("undo"))));
+	_undo_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString(wxT("undo"))));
 
-	_redo_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString("redo")));
-	_redo_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString("redo")));
-	_redo_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString("redo")));
+	_redo_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString(wxT("redo"))));
+	_redo_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString(wxT("redo"))));
+	_redo_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString(wxT("redo"))));
 
-	_record_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString("record")));
-	_record_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString("record")));
-	_record_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString("record")));
+	_record_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString(wxT("record"))));
+	_record_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString(wxT("record"))));
+	_record_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString(wxT("record"))));
 
-	_overdub_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString("overdub")));
-	_overdub_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString("overdub")));
-	_overdub_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString("overdub")));
+	_overdub_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString(wxT("overdub"))));
+	_overdub_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString(wxT("overdub"))));
+	_overdub_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString(wxT("overdub"))));
 
-	_multiply_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString("multiply")));
-	_multiply_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString("multiply")));
-	_multiply_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString("multiply")));
+	_multiply_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString(wxT("multiply"))));
+	_multiply_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString(wxT("multiply"))));
+	_multiply_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString(wxT("multiply"))));
 
-	_replace_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString("replace")));
-	_replace_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString("replace")));
-	_replace_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString("replace")));
+	_replace_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString(wxT("replace"))));
+	_replace_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString(wxT("replace"))));
+	_replace_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString(wxT("replace"))));
 
-	_insert_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString("insert")));
-	_insert_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString("insert")));
-	_insert_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString("insert")));
+	_insert_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString(wxT("insert"))));
+	_insert_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString(wxT("insert"))));
+	_insert_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString(wxT("insert"))));
 
-	_once_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString("oneshot")));
-	_once_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString("oneshot")));
-	_once_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString("oneshot")));
+	_once_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString(wxT("oneshot"))));
+	_once_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString(wxT("oneshot"))));
+	_once_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString(wxT("oneshot"))));
 
-	_trig_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString("trigger")));
-	_trig_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString("trigger")));
-	_trig_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString("trigger")));
+	_trig_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString(wxT("trigger"))));
+	_trig_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString(wxT("trigger"))));
+	_trig_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString(wxT("trigger"))));
 
 	_delay_button->pressed.connect (slot (*this, &LooperPanel::delay_button_press_event));
 	_delay_button->released.connect (slot (*this, &LooperPanel::delay_button_release_event));
-	_delay_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString("delay_trigger")));
+	_delay_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString(wxT("delay_trigger"))));
 
-	_reverse_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString("reverse")));
-	_reverse_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString("reverse")));
-	_reverse_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString("reverse")));
+	_reverse_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString(wxT("reverse"))));
+	_reverse_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString(wxT("reverse"))));
+	_reverse_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString(wxT("reverse"))));
 
-	_substitute_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString("substitute")));
-	_substitute_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString("substitute")));
-	_substitute_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString("substitute")));
+	_substitute_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString(wxT("substitute"))));
+	_substitute_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString(wxT("substitute"))));
+	_substitute_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString(wxT("substitute"))));
 	
-	_mute_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString("mute")));
-	_mute_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString("mute")));
-	_mute_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString("mute")));
+	_mute_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString(wxT("mute"))));
+	_mute_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString(wxT("mute"))));
+	_mute_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString(wxT("mute"))));
+
+	_pause_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString(wxT("pause"))));
+	_pause_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString(wxT("pause"))));
+	_pause_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString(wxT("pause"))));
+
+	_solo_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString(wxT("solo"))));
+	_solo_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString(wxT("solo"))));
+	_solo_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString(wxT("solo"))));
 
 	_halfx_button->pressed.connect (bind (slot (*this, &LooperPanel::rate_button_event), 0.5f));
 	_halfx_button->bind_request.connect (bind (slot (*this, &LooperPanel::rate_bind_events), 0.5f));
@@ -576,16 +683,16 @@ LooperPanel::bind_events()
 	_2x_button->pressed.connect (bind (slot (*this, &LooperPanel::rate_button_event), 2.0f));
 	_2x_button->bind_request.connect (bind (slot (*this, &LooperPanel::rate_bind_events), 2.0f));
 
-	_scratch_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString("scratch")));
-	_scratch_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString("scratch")));
-	_scratch_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString("scratch")));
+	_scratch_button->pressed.connect (bind (slot (*this, &LooperPanel::pressed_events), wxString(wxT("scratch"))));
+	_scratch_button->released.connect (bind (slot (*this, &LooperPanel::released_events), wxString(wxT("scratch"))));
+	_scratch_button->bind_request.connect (bind (slot (*this, &LooperPanel::button_bind_events), wxString(wxT("scratch"))));
 
-	_save_button->clicked.connect (bind (slot (*this, &LooperPanel::clicked_events), wxString("save")));
-	_load_button->clicked.connect (bind (slot (*this, &LooperPanel::clicked_events), wxString("load")));
+	_save_button->clicked.connect (bind (slot (*this, &LooperPanel::clicked_events), wxString(wxT("save"))));
+	_load_button->clicked.connect (bind (slot (*this, &LooperPanel::clicked_events), wxString(wxT("load"))));
 
 
-	_scratch_control->pressed.connect (bind (slot (*this, &LooperPanel::scratch_events), wxString("scratch_press")));
-	_scratch_control->released.connect (bind (slot (*this, &LooperPanel::scratch_events), wxString("scratch_release")));
+	_scratch_control->pressed.connect (bind (slot (*this, &LooperPanel::scratch_events), wxString(wxT("scratch_press"))));
+	_scratch_control->released.connect (bind (slot (*this, &LooperPanel::scratch_events), wxString(wxT("scratch_release"))));
 
 	
 	_loop_control->MidiBindingChanged.connect (slot (*this, &LooperPanel::got_binding_changed));
@@ -611,6 +718,8 @@ void LooperPanel::create_buttons()
  	_trig_button = new PixButton(this, ID_TrigButton);
  	_once_button = new PixButton(this, ID_OnceButton);
  	_mute_button = new PixButton(this, ID_MuteButton);
+ 	_pause_button = new PixButton(this, ID_PauseButton);
+ 	_solo_button = new PixButton(this, ID_SoloButton);
  	_scratch_button = new PixButton(this, ID_ScratchButton);
  	_halfx_button = new PixButton(this, ID_HalfXButton, true);
  	_1x_button = new PixButton(this, ID_OneXButton, true);
@@ -684,6 +793,18 @@ void LooperPanel::create_buttons()
 	_mute_button->set_disabled_bitmap (wxBitmap(mute_disabled));
 	_mute_button->set_active_bitmap (wxBitmap(mute_active));
 
+	_pause_button->set_normal_bitmap (wxBitmap(pause_normal));
+	_pause_button->set_selected_bitmap (wxBitmap(pause_selected));
+	_pause_button->set_focus_bitmap (wxBitmap(pause_focus));
+	_pause_button->set_disabled_bitmap (wxBitmap(pause_disabled));
+	_pause_button->set_active_bitmap (wxBitmap(pause_active));
+
+	_solo_button->set_normal_bitmap (wxBitmap(solo_normal));
+	_solo_button->set_selected_bitmap (wxBitmap(solo_selected));
+	_solo_button->set_focus_bitmap (wxBitmap(solo_focus));
+	_solo_button->set_disabled_bitmap (wxBitmap(solo_disabled));
+	_solo_button->set_active_bitmap (wxBitmap(solo_active));
+
 	_scratch_button->set_normal_bitmap (wxBitmap(scratch_normal));
 	_scratch_button->set_selected_bitmap (wxBitmap(scratch_selected));
 	_scratch_button->set_focus_bitmap (wxBitmap(scratch_focus));
@@ -736,57 +857,6 @@ void LooperPanel::create_buttons()
 }
 
 
-wxString
-LooperPanel::get_pixmap_path (const wxString & namebase)
-{
-	wxString filename;
-	wxString pixmapdir("pixmaps/");
-	
-#ifdef PIXMAPDIR
-	pixmapdir = PIXMAPDIR;
-#endif
-	
-	if (wxFile::Exists(wxString::Format(wxT("%s%s"), pixmapdir.c_str(), namebase.c_str()))) {
-		filename = wxString::Format(wxT("%s%s"), pixmapdir.c_str(), namebase.c_str());
-	}
-	else if (wxFile::Exists(wxString::Format(wxT("pixmaps%c%s"), wxFileName::GetPathSeparator(), namebase.c_str()))) {
-		filename = wxString::Format(wxT("pixmaps%c%s"), wxFileName::GetPathSeparator(), namebase.c_str());
-	}
-	else if (wxFile::Exists (namebase)) {
-		filename = namebase;
-	}
-	
-	return filename;
-}
-
-bool
-LooperPanel::load_bitmaps (PixButton * butt, wxString namebase)
-{
-	wxString bpath;
-
-	if(!(bpath = get_pixmap_path(namebase + wxT("_normal.png"))).empty()) {
-		butt->set_normal_bitmap (wxBitmap(bpath, wxBITMAP_TYPE_PNG));
-	}
-	
-	if(!(bpath = get_pixmap_path(namebase + wxT("_selected.png"))).empty()) {
-		butt->set_selected_bitmap (wxBitmap(bpath, wxBITMAP_TYPE_PNG));
-	}
-
-	if(!(bpath = get_pixmap_path(namebase + wxT("_focus.png"))).empty()) {
-		butt->set_focus_bitmap (wxBitmap(bpath, wxBITMAP_TYPE_PNG));
-	}
-
-	if(!(bpath = get_pixmap_path(namebase + wxT("_disabled.png"))).empty()) {
-		butt->set_disabled_bitmap (wxBitmap(bpath, wxBITMAP_TYPE_PNG));
-	}
-
-	if(!(bpath = get_pixmap_path(namebase + wxT("_active.png"))).empty()) {
-		butt->set_active_bitmap (wxBitmap(bpath, wxBITMAP_TYPE_PNG));
-	}
-
-	return true;
-}
-
 
 void
 LooperPanel::update_controls()
@@ -808,6 +878,10 @@ LooperPanel::update_controls()
 		_loop_control->get_value(_index, wxT("feedback"), val);
 		_feedback_control->set_value ((val * 100.0f));
 	}
+	if (_loop_control->is_updated(_index, wxT("input_gain"))) {
+		_loop_control->get_value(_index, wxT("input_gain"), val);
+		_in_gain_control->set_value (val);
+	}
 	if (_loop_control->is_updated(_index, wxT("rec_thresh"))) {
 		_loop_control->get_value(_index, wxT("rec_thresh"), val);
 		_thresh_control->set_value (val);
@@ -825,9 +899,10 @@ LooperPanel::update_controls()
 		}
 	}
 	else {
-		// use global in if we don't have discrete
-		_loop_control->get_global_value (wxT("in_peak_meter"), val);
-		_thresh_control->set_indicator_value (val);
+		if (_loop_control->is_updated(_index, wxT("in_peak_meter"))) {
+			_loop_control->get_value(_index, wxT("in_peak_meter"), val);
+			_thresh_control->set_indicator_value (val);
+		}
 	}
 
 	if (_loop_control->is_updated(_index, wxT("out_peak_meter"))) {
@@ -844,6 +919,14 @@ LooperPanel::update_controls()
 
 		update_rate_buttons(val);
 	}
+	if (_loop_control->is_updated(_index, wxT("stretch_ratio"))) {
+		_loop_control->get_value(_index, wxT("stretch_ratio"), val);
+		_stretch_control->set_value (val);
+	}
+	if (_loop_control->is_updated(_index, wxT("pitch_shift"))) {
+		_loop_control->get_value(_index, wxT("pitch_shift"), val);
+		_pitch_control->set_value (val);
+	}
 	if (_loop_control->is_updated(_index, wxT("rate_output"))) {
 		_loop_control->get_value(_index, wxT("rate_output"), val);
 		if (val < 0.0) {
@@ -858,6 +941,22 @@ LooperPanel::update_controls()
 		_scratch_control->set_value (val);
 		_scratch_control->set_indicator_value (val);
 	}
+
+	/*
+	if (_loop_control->is_updated(_index, wxT("output_latency"))) {
+		_loop_control->get_value(_index, wxT("output_latency"), val);
+		_outlatency_spin->set_value (val);
+	}
+	if (_loop_control->is_updated(_index, wxT("input_latency"))) {
+		_loop_control->get_value(_index, wxT("input_latency"), val);
+		_inlatency_spin->set_value (val);
+	}
+	if (_loop_control->is_updated(_index, wxT("trigger_latency"))) {
+		_loop_control->get_value(_index, wxT("trigger_latency"), val);
+		_triglatency_spin->set_value (val);
+	}
+	*/
+	
 // 	if (_loop_control->is_updated(_index, "quantize")) {
 // 		_loop_control->get_value(_index, "quantize", val);
 // 		//_quantize_choice->set_index_value ((int)val);
@@ -869,15 +968,24 @@ LooperPanel::update_controls()
 // 	}
 	if (_loop_control->is_updated(_index, wxT("sync"))) {
 		_loop_control->get_value(_index, wxT("sync"), val);
-		_sync_check->set_value (val > 0.0);
+		_sync_check->set_value (val > 0.0f);
 	}
 	if (_loop_control->is_updated(_index, wxT("playback_sync"))) {
 		_loop_control->get_value(_index, wxT("playback_sync"), val);
-		_play_sync_check->set_value (val > 0.0);
+		_play_sync_check->set_value (val > 0.0f);
 	}
 	if (_loop_control->is_updated(_index, wxT("use_feedback_play"))) {
 		_loop_control->get_value(_index, wxT("use_feedback_play"), val);
-		_play_feed_check->set_value (val > 0.0);
+		_play_feed_check->set_value (val > 0.0f);
+	}
+	if (_loop_control->is_updated(_index, wxT("tempo_stretch"))) {
+		_loop_control->get_value(_index, wxT("tempo_stretch"), val);
+		_tempo_stretch_check->set_value (val > 0.0f);
+	}
+
+	if (_loop_control->is_updated(_index, wxT("is_soloed"))) {
+		_loop_control->get_value(_index, wxT("is_soloed"), val);
+		_solo_button->set_active(val > 0.0f);
 	}
 
 	for (int i=0; i < _chan_count; ++i) {
@@ -891,7 +999,7 @@ LooperPanel::update_controls()
 	if (_use_main_in_check) {
 		if (_loop_control->is_updated(_index, wxT("use_common_ins"))) {
 			_loop_control->get_value(_index, wxT("use_common_ins"), val);
-			_use_main_in_check->set_value (val > 0.0);
+			_use_main_in_check->set_value (val > 0.0f);
 		}
 	}
 // 	if (_loop_control->is_updated(_index, "use_rate")) {
@@ -927,10 +1035,12 @@ LooperPanel::update_state()
 	wxString statestr, nstatestr;
 	LooperState state, nextstate;
 	float val;
-	
+	float soloed = false;
+
 	_loop_control->get_state(_index, state, statestr);
 	_loop_control->get_next_state(_index, nextstate, nstatestr);
 	_loop_control->get_value(_index, wxT("waiting"), val);
+	_loop_control->get_value(_index, wxT("is_soloed"), soloed);
 	_waiting = (val > 0.0f) ? true : false;
 	
 
@@ -976,6 +1086,9 @@ LooperPanel::update_state()
 	case LooperStateOneShot:
 		_once_button->set_active(false);
 		break;
+	case LooperStatePaused:
+		_pause_button->set_active(false);
+		break;
 	default:
 		break;
 	}
@@ -1027,6 +1140,10 @@ LooperPanel::update_state()
 		_once_button->set_active(true);
 		_flashing_button = _once_button;
 		break;
+	case LooperStatePaused:
+		_pause_button->set_active(true);
+		_flashing_button = _pause_button;
+		break;
 	default:
 		break;
 	}
@@ -1067,7 +1184,10 @@ LooperPanel::update_state()
 				break;
 			case LooperStateMuted:
 			case LooperStatePlaying:
-				if (state == LooperStatePlaying || state == LooperStateMuted) {
+				if (soloed) {
+					_flashing_button = _solo_button;
+				}
+				else if (state == LooperStatePlaying || state == LooperStateMuted) {
 					_flashing_button = _reverse_button;
 				}
 				break;
@@ -1078,7 +1198,12 @@ LooperPanel::update_state()
 		}
 		else if (state == LooperStatePlaying || state == LooperStateMuted) {
 			// special case, we are pending reverse
-			_flashing_button = _reverse_button;
+			if (soloed) {
+				_flashing_button = _solo_button;
+			} else {
+				_flashing_button = _reverse_button;
+			}
+			
 		}
 		
 		// make sure flash time is going
@@ -1092,12 +1217,14 @@ LooperPanel::update_state()
 			_flash_timer->Stop();
 
 			_loop_control->get_value(_index, wxT("rate_output"), val);
-			if (val < 0.0) {
+			if (val < 0.0f) {
 				_reverse_button->set_active(true);
 			}
 			else {
 				_reverse_button->set_active(false);
 			}
+
+			_solo_button->set_active(soloed);
 		}
 
 	}
@@ -1186,7 +1313,7 @@ LooperPanel::button_bind_events (wxString cmd)
 	
 	info.channel = 0;
 	info.type = "n";
-	snprintf(cmdbuf, sizeof(cmdbuf), "%s", cmd.c_str());
+	snprintf(cmdbuf, sizeof(cmdbuf), "%s", (const char *) cmd.ToAscii());
 	info.control = cmdbuf;
 
 	if (cmd == wxT("delay_trigger")) {
@@ -1195,8 +1322,8 @@ LooperPanel::button_bind_events (wxString cmd)
 		info.command = "note"; // should this be something else?
 	}
 	info.instance = _index;
-	info.lbound = 0.0;
-	info.ubound = 1.0;
+	info.lbound = 0.0f;
+	info.ubound = 1.0f;
 
 	start_learning (info);
 }
@@ -1223,7 +1350,7 @@ void
 LooperPanel::delay_button_press_event (int button)
 {
 	_tap_val *= -1.0f;
-	post_control_event (wxString("delay_trigger"), _tap_val);
+	post_control_event (wxString(wxT("delay_trigger")), _tap_val);
 }
 
 void
@@ -1231,7 +1358,7 @@ LooperPanel::delay_button_release_event (int button)
 {
 	if (button == PixButton::MiddleButton) {
 		_tap_val *= -1.0f;
-		post_control_event (wxString("delay_trigger"), _tap_val);
+		post_control_event (wxString(wxT("delay_trigger")), _tap_val);
 	}
 }
 
@@ -1244,7 +1371,7 @@ LooperPanel::rate_button_event (int button, float rate)
 // 	val = val == 0.0f ? 1.0f : 0.0f;
 // 	post_control_event (wxString("use_rate"), val);
 
-	post_control_event (wxString("rate"), rate);
+	post_control_event (wxString(wxT("rate")), rate);
 	update_rate_buttons (rate);
 	_rate_control->set_value (rate);
 }
@@ -1254,7 +1381,7 @@ LooperPanel::clicked_events (int button, wxString cmd)
 {
 	if (cmd == wxT("save"))
 	{
-		wxString filename = ::wxGetApp().getFrame()->get_keyboard().do_file_selector (wxT("Choose file to save loop"),
+		wxString filename = _mainpanel->do_file_selector (wxT("Choose file to save loop"),
 											      wxT("wav"), wxT("*.*"),  wxSAVE|wxCHANGE_DIR|wxOVERWRITE_PROMPT);
 		
 		if ( !filename.empty() )
@@ -1269,7 +1396,7 @@ LooperPanel::clicked_events (int button, wxString cmd)
 	}
 	else if (cmd == wxT("load"))
 	{
-		wxString filename = ::wxGetApp().getFrame()->get_keyboard().do_file_selector(wxT("Choose file to open"), wxT(""), wxT("*.*"), wxOPEN|wxCHANGE_DIR);
+		wxString filename = _mainpanel->do_file_selector(wxT("Choose file to open"), wxT(""), wxT("*.*"), wxOPEN|wxCHANGE_DIR);
 		
 		if ( !filename.empty() )
 		{
@@ -1294,6 +1421,10 @@ LooperPanel::slider_events(float val, int id)
 		ctrl = wxT("feedback");
 		val = _feedback_control->get_value() / 100.0f;
 		break;
+	case ID_InputGainControl:
+		ctrl = wxT("input_gain");
+		val = _in_gain_control->get_value();
+		break;
 	case ID_DryControl:
 		ctrl = wxT("dry");
 		val = _dry_control->get_value();
@@ -1311,6 +1442,26 @@ LooperPanel::slider_events(float val, int id)
 		val = _rate_control->get_value();
 		update_rate_buttons (val);
 		break;
+	case ID_StretchControl:
+		ctrl = wxT("stretch_ratio");
+		val = _stretch_control->get_value();
+		break;
+	case ID_PitchControl:
+		ctrl = wxT("pitch_shift");
+		val = _pitch_control->get_value();
+		break;
+	case ID_OutputLatency:
+		ctrl = wxT("output_latency");
+		cerr << "outlat " << val << endl;
+		break;
+	case ID_InputLatency:
+		ctrl = wxT("input_latency");
+		cerr << "inlat " << val << endl;
+		break;
+	case ID_TriggerLatency:
+		ctrl = wxT("trigger_latency");
+		cerr << "triglat " << val << endl;
+		break;
 	default:
 		break;
 	}
@@ -1330,8 +1481,8 @@ LooperPanel::control_bind_events(int id)
 	info.type = "cc";
 	info.command = "set";
 	info.instance = _index;
-	info.lbound = 0.0;
-	info.ubound = 1.0;
+	info.lbound = 0.0f;
+	info.ubound = 1.0f;
 
 	
 	switch(id)
@@ -1348,6 +1499,10 @@ LooperPanel::control_bind_events(int id)
 		info.control = "dry";
 		info.style = MidiBindInfo::GainStyle;
 		break;
+	case ID_InputGainControl:
+		info.control = "input_gain";
+		info.style = MidiBindInfo::GainStyle;
+		break;
 	case ID_WetControl:
 		info.control = "wet";
 		info.command = "set";
@@ -1363,6 +1518,18 @@ LooperPanel::control_bind_events(int id)
 		info.lbound = 0.25;
 		info.ubound = 4.0;
 		break;
+	case ID_StretchControl:
+		info.control = "stretch_ratio";
+		info.style = MidiBindInfo::NormalStyle;
+		info.lbound = 0.5;
+		info.ubound = 4.0;
+		break;
+	case ID_PitchControl:
+		info.control = "pitch_shift";
+		info.style = MidiBindInfo::IntegerStyle;
+		info.lbound = -12;
+		info.ubound = 12;
+		break;
 	case ID_SyncCheck:
 		info.control = "sync";
 		info.style = MidiBindInfo::NormalStyle;
@@ -1375,6 +1542,10 @@ LooperPanel::control_bind_events(int id)
 		info.control = "use_feedback_play";
 		info.style = MidiBindInfo::NormalStyle;
 		break;
+	case ID_TempoStretchCheck:
+		info.control = "tempo_stretch";
+		info.style = MidiBindInfo::NormalStyle;
+		break;
 	default:
 		donothing = true;
 		break;
@@ -1401,8 +1572,8 @@ void LooperPanel::pan_bind_events(int chan)
 	info.type = "cc";
 	info.command = "set";
 	info.instance = _index;
-	info.lbound = 0.0;
-	info.ubound = 1.0;
+	info.lbound = 0.0f;
+	info.ubound = 1.0f;
 	info.style = MidiBindInfo::NormalStyle;
 
 	snprintf (cmdbuf, sizeof(cmdbuf), "pan_%d", chan+1);
